from .mvf1 import MultiViewerForF1
from .mvf1 import MultiViewerForF1Error
from .mvf1 import Player
from .mcp import mcp

__title__ = "mvf1"
__version__ = "2.0.0"
__author__ = "Rob Spectre"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Rob Spectre"
